# Configurations
lora_params = False
lora_adr = True                 # ADR enable/disable
lora_dr = 2                     # 0... 5
lora_send_trials = 3            # 1... 10
lora_join_trials = 1            # 1... 3
lora_port = 1                   # 1... 223
lora_cnf = True                 # confirmed messages enable/disable
lora_heartbeat = 48             # 0... 65535‬
lora_interval = 7               # 5... 65535‬

periphery_params = False
periphery_piezo_mode = 2        # 0 (Off), 1 (1 tone), 2 (2 tones), 3 (2*dur1), 4 (4*dur1), 5 (2*dur2), 6 (4*dur2), 7 (2*dur1 & 2*dur2), 8 (4*dur1 & 4*dur2)
periphery_piezo_freq1 = 1       # 0... 3 -> f1: 0 (500Hz), 1 (1kHz), 2 (2kHz), 3 (4kHz)
periphery_piezo_freq2 = 2       # 0... 3 -> f2: 0 (500Hz), 1 (1kHz), 2 (2kHz), 3 (4kHz)
periphery_leds_enable = 0x0F    # LED EN bits: 0b0000'1111
periphery_accel_mode = 0        # 0... 6
periphery_show_hourglass = True

image_params = False
image_epd_mode = 1              # 0 = show all in the memory, 1 = show selected, 2 = toggle selected
image_codes = [0x001D, 0x001E, 0x001F]

user_text_params = False
user_text_x_pos = 8             # 0... 199
user_text_y_pos = 8             # 0... 199
user_text_font_size = 24        # 8, 12, 16, 20, 24
user_text_state = 0             # 0 (0°), 1 (0° inverted), 2 (90°), 3 (90° inverted), 4 (180°), 5 (180° inverted), 6 (270°), 7 (270° inverted)
user_text_chars = "123\nABC"

#########################################################################
print("--> OxobuttonQT LoRa Downlink generator <--\n")
downlink_message = ""
if lora_params:
    print("generating the lora_params...")
    downlink_message += "B0"
    downlink_message += "01" if lora_adr else "00"
    if lora_dr < 0 or lora_dr > 5: print("-> invalid LoRa DR!")
    downlink_message += ''.join(format(lora_dr, '02X'))
    if lora_send_trials < 1 or lora_send_trials > 10: print("-> invalid LoRa send trials!")
    downlink_message += ''.join(format(lora_send_trials, '02X'))
    if lora_join_trials < 1 or lora_join_trials > 3: print("-> invalid LoRa join trials!")
    downlink_message += ''.join(format(lora_join_trials, '02X'))
    if lora_port < 1 or lora_port > 223: print("-> invalid LoRa port!")
    downlink_message += ''.join(format(lora_port, '02X'))
    downlink_message += "01" if lora_cnf else "00"
    if lora_heartbeat < 0 or lora_heartbeat > 65535: print("-> invalid LoRa heartbeat!")
    downlink_message += ''.join(format(lora_heartbeat, '04X'))
    if lora_interval < 5 or lora_interval > 65535: print("-> invalid LoRa interval!")
    downlink_message += ''.join(format(lora_interval, '04X'))

if periphery_params:
    print("generating the periphery_params...")
    downlink_message += "B1"
    if periphery_piezo_mode < 0 or periphery_piezo_mode > 15:  print("-> invalid piezo mode!")
    downlink_message += ''.join(format(periphery_piezo_mode, '01X'))
    if periphery_piezo_freq1 < 0 or periphery_piezo_freq1 > 3:  print("-> invalid piezo frequency1!")
    if periphery_piezo_freq2 < 0 or periphery_piezo_freq2 > 3:  print("-> invalid piezo frequency2!")
    periphery_piezo_freq = (periphery_piezo_freq2 << 2) | periphery_piezo_freq1
    downlink_message += ''.join(format(periphery_piezo_freq, '01X'))
    downlink_message += "00"    # reserved
    if periphery_leds_enable < 0 or periphery_leds_enable > 0x0F:  print("-> invalid LEDs enable!")
    downlink_message += ''.join(format(periphery_leds_enable, '02X'))
    if periphery_accel_mode < 0 or periphery_accel_mode > 6:  print("-> invalid accel mode!")
    downlink_message += ''.join(format(periphery_accel_mode, '02X'))
    downlink_message += "01" if periphery_show_hourglass else "00"

if image_params:
    print("generating the image_params...")
    downlink_message += "B2"
    if image_epd_mode < 0 or image_epd_mode > 2:  print("-> invalid epd mode!")
    downlink_message += ''.join(format(image_epd_mode, '02X'))
    if image_epd_mode != 0:
        downlink_message += ''.join(format(len(image_codes), '02X'))
        for image_code in image_codes:
            downlink_message += ''.join(format(image_code, '04X'))

if user_text_params:
    print("generating the user_text_params...")
    downlink_message += "B3"
    if user_text_x_pos < 0 or user_text_x_pos > 199: print("-> invalid text position x!")
    downlink_message += ''.join(format(user_text_x_pos, '02X'))
    if user_text_y_pos < 0 or user_text_y_pos > 199: print("-> invalid text position y!")
    downlink_message += ''.join(format(user_text_y_pos, '02X'))
    if user_text_font_size < 8 or user_text_font_size > 24: print("-> invalid text font size!")
    downlink_message += ''.join(format(user_text_font_size, '02X'))
    if user_text_state < 0 or user_text_state > 7: print("-> invalid text state!")
    downlink_message += ''.join(format(user_text_state, '02X'))
    downlink_message += ''.join(format(len(user_text_chars), '02X'))
    for char in user_text_chars:
        downlink_message += ''.join(format(ord(char), '02X'))

print("\nFinal LoRa downlink: ==> " + downlink_message + " <== (" + str(len(downlink_message)//2) + "/50 bytes)")
if len(downlink_message)//2 > 50: print("-> Error: Too many bytes!")
print("")
